package com.yihu.hos.rest.common.format;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.yihu.ehr.dbhelper.jdbc.DBHelper;
import com.yihu.hos.core.datatype.DateUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.rest.controllers.DataCollectController;
import com.yihu.hos.rest.models.crawler.adapter.AdapterDataSet;
import com.yihu.hos.rest.models.crawler.adapter.AdapterDict;
import com.yihu.hos.rest.models.crawler.adapter.AdapterMetaData;
import com.yihu.hos.rest.models.crawler.adapter.StdDataSet;
import com.yihu.hos.rest.models.crawler.patient.Patient;
import com.yihu.hos.rest.models.crawler.transform.DictDataType;
import com.yihu.hos.rest.models.crawler.transform.TransformType;
import com.yihu.hos.rest.models.standard.standard.StdMetaDataModel;
import com.yihu.hos.web.framework.constant.QcResultConstant;
import com.yihu.hos.web.framework.constant.SqlConstants;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * 目前只处理json格式
 * <p>
 * json 格式
 * {
 * "inner_version":"xxxxx",
 * "patient_id":"xxxx",
 * "event_no":"xxxx",
 * "code":"dataset_code",
 * "org_code":"xxxx"
 * "data":
 * [{"metadata_code1":"5","metadata_code2":"6"},
 * [{"metadata_code1":"1","metadata_code2":"2"}]}
 * <p>
 * Created by Air on 2015/6/4.
 */
public class DataSetTransformer implements IDataTransformer {
    private JsonNode jsonObject;
    private Patient patient;
    private static DBHelper db;
    private JsonNode qcObject;

    private static Logger logger = LoggerFactory.getLogger(DataSetTransformer.class);

    public JsonNode getQcObject() {
        return qcObject;
    }

    public JsonNode getJsonObject() {
        return jsonObject;
    }

    public DataSetTransformer() {
        if (db == null) {
            db = new DBHelper();
        }
    }

    public boolean transfer(Map<String, AdapterDataSet> dataSetMap) {
        if (jsonObject != null) {
            String dataSetCode = jsonObject.get("code").asText();
            ArrayNode jsonArray = (ArrayNode) jsonObject.get("data");
            AdapterDataSet adapterDataSet = dataSetMap.get(dataSetCode);
            boolean transfer = transferJson(jsonArray, adapterDataSet);

            return transfer;
        }

        return false;
    }

    /**
     * 转换时间格式为utc、
     *
     * @param dataSetMap
     * @return
     */
    public boolean transferTimeFormat(Map<String, StdDataSet> dataSetMap) {
        if (jsonObject != null) {
            try {
                String dataSetCode = jsonObject.get("code").asText();
                ArrayNode jsonArray = (ArrayNode) jsonObject.get("data");
                StdDataSet stdDataSet = dataSetMap.get(dataSetCode);
                Map<Long, Map<String, StdMetaDataModel>> stdDataSetMetasMap = stdDataSet.getStdDataSetMetasMap();
                Map<String, StdMetaDataModel> stdMetaDataModelMap = stdDataSetMetasMap.get(stdDataSet.getStdDataSetId());

                for (Object objectRow : jsonArray) {
                    if (objectRow instanceof JsonNode) {
                        ObjectNode jsonObject = (ObjectNode) objectRow;
                        Iterator<Map.Entry<String, JsonNode>> fields = jsonObject.fields();
                        while (fields.hasNext()) {
                            Map.Entry<String, JsonNode> next = fields.next();
                            String key = next.getKey();
                            JsonNode jsonNode = next.getValue();
                            String value = jsonNode.asText();
                            String stdValue = transferElemTime(stdMetaDataModelMap, key, value);
                            if (stdValue==null){
                                stdValue="";
                            }
                            jsonObject.put(key, stdValue);

//                            if (jsonNode instanceof ObjectNode) {
//                                ObjectNode objectNode = (ObjectNode) next;
//                                objectNode.put(key, stdValue);
//                            }
                        }
                    }
                }
                return true;
            } catch (Exception e) {
                e.printStackTrace();
            }

            return false;
        }

        return false;
    }

    public String getData() {
        //确保文档有版本信息
        return jsonObject.asText();
    }

    public void setData(JsonNode data) {
        jsonObject = data;
        qcObject = data;
        setPatient();
    }

    @Override
    public Patient getPatient() {
        return patient;
    }

    public TransformType getTransformType() {
        return TransformType.DATA_SET_JSON;
    }

    /**
     * json 格式
     * {
     * "inner_version":"xxxxx",
     * "patient_id":"xxxx",
     * "event_no":"xxxx",
     * "code":"dataset_code",
     * "data":
     * [{"metadata_code1":"5","metadata_code2":"6"},
     * [{"metadata_code1":"1","metadata_code2":"2"}]}
     *
     * @param jsonArray
     * @param adapterDataSet
     * @return
     */
    public boolean transferJson(ArrayNode jsonArray, AdapterDataSet adapterDataSet) {
        for (Object objectRow : jsonArray) {
            if (objectRow instanceof JsonNode) {
                transferJsonRow((ObjectNode) objectRow, adapterDataSet);
            }
        }

        return false;
    }

    public void transferJsonRow(ObjectNode jsonObject, AdapterDataSet adapterDataSet) {
        Iterator<Map.Entry<String, JsonNode>> fields = jsonObject.fields();

        while (fields.hasNext()) {
            Map.Entry<String, JsonNode> next = fields.next();
            String key = next.getKey();
            JsonNode jsonNode = next.getValue();
            String value = jsonNode.asText();
            String stdValue = transferElem(adapterDataSet, key, value);
            jsonObject.put(key, stdValue);

//            if (jsonNode instanceof ObjectNode) {
//                ObjectNode objectNode = (ObjectNode) next;
//                objectNode.put(key, stdValue);
//            }
        }
    }

    /**
     * @param adapterDataSet 数据集编码
     * @param code           数据元编码
     * @param esbData        数据
     * @return String 标准值
     * @modify 2015.09.16 airhead 增加值与编码转换
     */
    public String transferElem(AdapterDataSet adapterDataSet, String code, String esbData) {
        Map<String, AdapterMetaData> adapterMetaDataMap = adapterDataSet.getAdapterMetaDataMap();
        AdapterMetaData adapterMetaData = adapterMetaDataMap.get(code);
        if (adapterMetaData==null){
            return esbData;
        }
        AdapterDict adapterDict = adapterMetaData.getAdapterDict();

        if (adapterDict == null) {
            return esbData;
        }
        String ehrData = null;
        DictDataType adapterDictDataType = adapterMetaData.getAdapterDictDataType();
        if (adapterDictDataType == DictDataType.VALUE) {
            ehrData = adapterDict.getAdapterValueToCodeMap().get(esbData) ==null? esbData: adapterDict.getAdapterValueToCodeMap().get(esbData);
        } else if (adapterDictDataType == DictDataType.CODE) {
            ehrData = adapterDict.getAdapterCodeToCodeMap().get(esbData) == null ? esbData : adapterDict.getAdapterCodeToCodeMap().get(esbData);
        }
        if (StringUtil.isEmpty(ehrData)) {
            return SqlConstants.EMPTY;
        }
        return ehrData;
    }

    public String transferElemTime(Map<String, StdMetaDataModel> stdMetaDataModelMap, String code, String esbData) {
        StdMetaDataModel stdMetaData = stdMetaDataModelMap.get(code);
        logger.info("原数据元key："+code +",进入transferElemTime。。。。,数据元数量："+stdMetaDataModelMap.keySet().size());
        if (stdMetaData == null ||stdMetaData.getColumnType() == null) {
            return esbData;
        }
        logger.info(esbData+" 时间转换开始11111111,数据元集合："+ stdMetaDataModelMap.keySet().toArray().toString());
        String columnType = stdMetaData.getColumnType();
        logger.info(esbData+" 时间转换开始222222222"+ code);
        try{
            if ("DATE".equals(columnType) || "DATETIME".equals(columnType)) {
                //时间转换
                logger.info(esbData+" 时间转换开始"+ code);
                if (!StringUtil.isEmpty(esbData)){
                    logger.info(esbData+" 时间转换开始1111");
                    String utcDate = DateUtil.getUtcDate(esbData, DateUtil.DEFAULT_YMDHMSDATE_FORMAT);
                    logger.info(esbData+" 时间转换结束："+utcDate);
                    return utcDate;
                }
            } else {
                return esbData;
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 根据DataSet信息构造Patient
     */
    private void setPatient() {
        patient = new Patient();
        patient.setPatientId(jsonObject.get("patient_id").asText());
        patient.setEventNo(jsonObject.get("event_no").asText());
        patient.setOrgCode(jsonObject.get("org_code").asText());
    }


    /* *************************** 以下为质控规则逻辑相关 **************************** add by hzy */

    public boolean genQcData(Map<String, AdapterDataSet> dataSetMap,String checkType) {
        if (jsonObject != null) {
            String dataSetCode = jsonObject.get("code").asText();
            ArrayNode jsonArray = (ArrayNode) jsonObject.get("data");
            AdapterDataSet adapterDataSet = dataSetMap.get(dataSetCode);
            //TODO 根据条件判断执行的规则
            switch (checkType){
                case  QcResultConstant.VALUE_EMPTY :checkDataIsNull(jsonArray,checkType);
                    break;
                case  QcResultConstant.VALUE_ERROR :checkDataTransfer(jsonArray, adapterDataSet, checkType);
                    break;
                default:
                    break;
            }


            return true;
        }

        return false;
    }


    //检测医院数据为空
    public void checkDataIsNull(ArrayNode jsonArray,String checkType) {
        ObjectMapper objectMapper = new ObjectMapper();
        ArrayNode arrayNode = objectMapper.createArrayNode();
        int realCount=0;
        for (Object objectRow : jsonArray) {
            if (objectRow instanceof JsonNode) {
                ObjectNode jsonObject = (ObjectNode) objectRow;
                Iterator<Map.Entry<String, JsonNode>> fields = jsonObject.fields();
                ObjectNode objectNode = objectMapper.createObjectNode();
                objectNode.put("total",jsonObject.size());
                while (fields.hasNext()) {
                    Map.Entry<String, JsonNode> next = fields.next();
                    String key = next.getKey();
                    JsonNode jsonNode = next.getValue();
                    String value = jsonNode.asText();
                    if (!StringUtil.isEmpty(value)) {
                        //医院数据是空
//                        jsonObject.remove(key);
                        logger.info("value:" + value);
                    }else {
                        realCount++;
                        objectNode.put(key,checkType);
                    }
                }
                objectNode.put("error",realCount);
                arrayNode.add(objectNode);
                realCount=0;
            }
        }
        jsonArray.removeAll();
        jsonArray.addAll(arrayNode);
    }

    //检测是否转换异常
    public void checkDataTransfer(ArrayNode jsonArray, AdapterDataSet adapterDataSet,String checkType) {
        ObjectMapper objectMapper = new ObjectMapper();
        ArrayNode arrayNode = objectMapper.createArrayNode();
        int realCount=0;
        for (Object objectRow : jsonArray) {
            if (objectRow instanceof JsonNode) {
                ObjectNode jsonObject = (ObjectNode) objectRow;
                Iterator<Map.Entry<String, JsonNode>> fields = jsonObject.fields();
                ObjectNode objectNode = objectMapper.createObjectNode();
                objectNode.put("total",jsonObject.size());
                while (fields.hasNext()) {
                    Map.Entry<String, JsonNode> next = fields.next();
                    String key = next.getKey();
                    JsonNode jsonNode = next.getValue();
                    String value = jsonNode.asText();
                    String stdValue = transferElem(adapterDataSet, key, value);
                    if (!StringUtil.isEmpty(value) && StringUtil.isEmpty(stdValue)) {
                        //转换后数据异常
                        objectNode.put(key, checkType+": org_value: " + value + " stdValue: " + stdValue);
                        logger.info("value: " + value + " stdValue:" + stdValue);
                        realCount++;
                    }
                }
                objectNode.put("error",realCount);
                arrayNode.add(objectNode);
                realCount=0;
            }
        }
        jsonArray.removeAll();
        jsonArray.addAll(arrayNode);
    }


}
