package com.yihu.hos.rest.services.crawler;

import com.yihu.hos.rest.models.crawler.config.SysConfig;
import com.yihu.hos.rest.models.crawler.patient.Patient;

import java.util.UUID;

/**
 * 自控文件索引类,用于生成文件路径,不确保文件路径存在
 *
 * @author Air
 * @version 1.0
 * @created 2015.07.01 18:06
 */
public class QCDataIndex {
    private Patient patient;
    private String createDate;

    public QCDataIndex(Patient patient,String createDate) {
        this.patient = patient;
        this.createDate = createDate;
    }

    public String getDirectory() {
        String dir = SysConfig.getInstance().getTempFile();
        return dir + "/qc_data/" + patient.getOrgCode() + "/" + patient.getOrgCode() + "-" + patient.getPatientId() + "-" + patient.getEventNo();
    }

    /**
     * 生成自控数据目录
     * orgCode-pateintId-eventNo/data
     *
     * @return
     */
    public String getDataDirectory() {
        return getDirectory() + "/" + IndexType.DATA;
    }

    public String createIndex(String indexType, String fileType) {
        UUID uuid = UUID.randomUUID();
        String index = uuid.toString();
        String dir = getDirectory() + "/" + indexType;
        return dir + "/" + index + fileType;
    }

    public String createDataSetIndex(String fileType) {
        String index = "qc_datasets";
        String dir = getDirectory() + "/" + IndexType.DATA;

        return dir + "/" + index + fileType;
    }


    /**
     * 生成最终档案包目录
     * qc_data/empty
     * qc_data/error
     * qc_data/other
     *
     * @param indexType
     * @param fileType
     * @return
     */
    public String createDataIndex(String indexType, String fileType) {
        return createIndex(IndexType.DATA + "/" + indexType, fileType);
    }

    /**
     * 质控-数据集实收应收情况
     *
     * @param fileType
     * @return
     */
    public String createQcDataSetIndex(String fileType) {
        return createDataSetIndex(fileType);
    }


    /* ************************* 日报包路径 *************************** */

    //基础目录
    public String getDailyDir() {
        String dir = SysConfig.getInstance().getTempFile();
        return dir + "/qc_daliy/" + createDate ;
    }

    public String getDataDailyDir() {
        return getDailyDir() + "/" + IndexType.DATA;
    }

    /**
     *  data 文件目录
     * @param fileType
     * @return
     */
    public String createQcDaliyIndex(String fileType) {
        String index = "qc_events";
        String dir = getDailyDir()+ "/" + IndexType.DATA;
        return dir + "/" + index + fileType;
    }


    /**
     * 生成日报质控包目录
     *
     * @param indexType
     * @param fileType
     * @return
     */
    public String createQcDailyIndex(String indexType, String fileType) {
        return createDailyIndex( indexType, fileType);
    }

    public String createDailyIndex(String indexType, String fileType) {
        UUID uuid = UUID.randomUUID();
        String index = uuid.toString();
        String dir = getDailyDir() + "/" + indexType;
        return dir + "/" + index + fileType;
    }


    public class FileType {
        public final static String XML = ".xml";
        public final static String JSON = ".json";
        public final static String ZIP = ".zip";
    }

    public class IndexType {
        public final static String QC_TYPE_DATASET = "1";//质控包类型-数据集
        public final static String QC_TYPE_DAILY = "2";//质控包类型-日报
        public final static String DATA = "data";   //病人档案数据目录
        public final static String CDA = "cda";     //病人cda档案目录
        public final static String STANDARD = "standard";   //病人标准档案目录
        public final static String ORIGIN = "origin";   //病人原始档案目录
        public final static String ZIP = "zip";         //病人压缩包目录
        public final static String DOCUMENT = "document";
    }
}
